package bookExamples.ch26Graphics.draw2d;

import gui.ClosableJFrame;
import gui.In;
import gui.run.RunMenu;
import gui.run.RunMenuBar;
import gui.run.RunMenuItem;
import j2d.ImageUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;

// randeep  - ImageShape
// ed - SquareShape
// gino - ConeShape
// dave - PentagonShape
// keith - RightTriangleShape
// damian - HexagonShape
// ian - CylinderShape
// enrique - EquilateralShape

public class JDrawComponent extends JComponent
        implements MouseListener, MouseMotionListener {
    Font font = new Font("Times", Font.PLAIN, 12);

    private Shapes shapeList = new Shapes(0, 0, 0, 0);
    private int x1, y1;
    private Point p2 = new Point();
    private Point lastPoint = new Point();
    private Shape lastSelectedShape = new Line2d(0, 0, 0, 0);


    public JDrawComponent() {
        addMouseMotionListener(this);
        addMouseListener(this);
    }


    private void ray() {
        shapeList.addLineToLastCircle();
        repaint();
    }


    private void erase() {
        shapeList = new Shapes(0, 0, 0, 0);
        repaint();
    }


    private RunMenuBar getMenuBar() {
        RunMenuBar mb = new RunMenuBar();
        mb.add(getFileMenu());
        mb.add(getColorMenu());
        mb.add(getDrawMenu());
        return mb;
    }

    private void outputToPrinter() {
        ImageUtils.print(this);
    }

    private RunMenu getDrawMenu() {
        RunMenu drawMenu = new RunMenu("[Draw");
        drawMenu.add(new RunMenuItem("[lens{alt control L}") {
            public void run() {
                lastSelectedShape = new Lens2d(x1, y1, p2.x, p2.y);
            }
        });

        drawMenu.add(new RunMenuItem("setRotationTo90") {
            public void run() {

                AffineTransform at = lastSelectedShape.getAffineTransform();

                at.setToRotation(90 * Math.PI / 180, x1, y1);
            }
        });
        drawMenu.add(new RunMenuItem("[erase{control E}") {
            public void run() {
                erase();
            }
        });
        drawMenu.add(new RunMenuItem("[line{control L}") {
            public void run() {
                lastSelectedShape = new Line2d(0, 0, 0, 0);
            }
        });
        //
        drawMenu.add(new RunMenuItem("R[hombus2d{control H}") {
            public void run() {
                lastSelectedShape = new Rhombus2d(x1, y1, p2.x, p2.y);
            }
        });
        drawMenu.add(new RunMenuItem("Pentagon[5{control 5}") {
            public void run() {
                lastSelectedShape = new Pentagon(x1, y1, p2.x, p2.y);
            }
        });
        drawMenu.add(new RunMenuItem("Sq[uareShape2{control U}") {
            public void run() {
                lastSelectedShape = new SquareShape2(x1, y1, p2.x, p2.y);
            }
        });
        drawMenu.add(new RunMenuItem("RtTrian[gle2d{alt T}") {
            public void run() {
                lastSelectedShape = new RtTriangle2d(x1, y1, p2.x, p2.y);
            }
        });
        drawMenu.add(new RunMenuItem("C[ylinder{control Y}") {
            public void run() {
                lastSelectedShape = new Cylinder(x1, y1, p2.x, p2.y);
            }
        });
        drawMenu.add(new RunMenuItem("[Cone{control K}") {
            public void run() {
                lastSelectedShape = new ConeShape(x1, y1, p2.x, p2.y);
            }
        });
        drawMenu.add(new RunMenuItem("[ImageShape{control I}") {
            public void run() {
                lastSelectedShape = new ImageShape(x1, y1, p2.x, p2.y);
            }
        });
        drawMenu.add(new RunMenuItem("[triangle{control T}") {
            public void run() {
                lastSelectedShape = new Triangle2d(0, 0, 0, 0);
            }
        });
        drawMenu.add(new RunMenuItem("[circle{control C}") {
            public void run() {
                lastSelectedShape = new Circle2d(0, 0, 0, 0);
            }
        });
        drawMenu.add(new RunMenuItem("[oval{control O}") {
            public void run() {
                lastSelectedShape = new Oval2d(0, 0, 0, 0);
            }
        });
        drawMenu.add(new RunMenuItem("[rect{control 6}") {
            public void run() {
                lastSelectedShape = new Rect2d(0, 0, 0, 0);
            }
        });
        drawMenu.add(new RunMenuItem("[FillRect2d{control F}") {
            public void run() {
                lastSelectedShape = new FillRect2d(0, 0, 0, 0);
            }
        });
        drawMenu.add(new RunMenuItem("[string{control s}") {
            public void run() {
                lastSelectedShape = new String2d(p2.x, p2.y, "String me!");
            }
        });
        drawMenu.add(new RunMenuItem("r[ay{control R}") {
            public void run() {
                ray();
            }
        });
        return drawMenu;
    }

    private RunMenu getColorMenu() {
        RunMenu colorMenu = new RunMenu("[Color");
        colorMenu.add(new RunMenuItem("set [background color{alt control c}") {
            public void run() {
                setBackgroundColor();
            }
        });
        colorMenu.add(new RunMenuItem("set [foreground color{alt control f}") {
            public void run() {
                setForegroundColor();
            }
        });
        colorMenu.add(new RunMenuItem("set [last selected color{alt control s}") {
            public void run() {
                lastSelectedShape.setColor(In.getColor());
            }
        });
        return colorMenu;
    }

    private void setForegroundColor() {
        getParent().setForeground(In.getColor());
    }

    private void setBackgroundColor() {
        getParent().setBackground(In.getColor());
    }

    private RunMenu getFileMenu() {
        RunMenu fileMenu = new RunMenu("[File");

        fileMenu.add(new RunMenuItem("[print{control p}") {
            public void run() {
                outputToPrinter();
            }
        });

        fileMenu.add(new RunMenuItem("[quit{control q}") {
            public void run() {
                System.exit(0);
            }
        });


        return fileMenu;
    }


    public void mouseDragged(MouseEvent e) {
        e.consume();
        lastPoint.x = p2.x;
        lastPoint.y = p2.y;
        p2.x = e.getX();
        p2.y = e.getY();
        repaint();
    }

    public void mouseMoved(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        e.consume();
        x1 = e.getX();
        y1 = e.getY();
        int x21 = -1;
        p2.x = x21;
    }

    public void mouseReleased(MouseEvent e) {
        e.consume();
        int x2 = e.getX();
        int y2 = e.getY();
        shapeList.add(lastSelectedShape.getInstance(x1, y1, x2, y2));
        int xl1 = -1;
        lastPoint.x = xl1;
        int x21 = -1;
        p2.x = x21;
        repaint();
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void drawRubberBand(Graphics g) {
        if (p2.x == -1) return;
        g.setXORMode(getBackground());
        lastSelectedShape.getInstance(x1, y1, p2.x, p2.y).paint(g);
    }

    public void paint(Graphics g) {
        g.setFont(font);
        shapeList.paint(g);
        drawRubberBand(g);
    }

    public static void main(String args[]) {
        JDrawComponent drawComponent = new JDrawComponent();
        ClosableJFrame cf = new ClosableJFrame("DocJavaDraw");
        Container c = cf.getContentPane();
        c.add(drawComponent);

        cf.setJMenuBar(drawComponent.getMenuBar());
        cf.setSize(300, 300);
        cf.setVisible(true);
    }


}













